import java.awt.*;
import java.awt.event.*;
import gl4java.*;
import gl4java.utils.glut.*;
import gl4java.GLContext;
import gl4java.awt.GLAnimCanvas;

import java.util.Vector;
import java.util.Iterator;

/**
 * The joogl canvas object.   The window that is rendered into.
 * <p>
 * Create a canvas that has a scene graph rooted at it.  This is a subclass of
 * GLAnimCanvas, and uses the facilities of the AnimCanvas to render.
 * <p>
 * The canvas contains one or more jooglRoot nodes that are
 * the roots of scene graphs to be drawn in viewports on the canvas.
 * The graph attached to each root node is drawn in the canvas 
 * using the specified viewport, and the specified camera object for viewing.  
 * If the camera appears in the scene graph, it will be transformed
 * accordingly, but it does not need to be attached to the graph (in which case
 * it will use its parameters directly).
 * <p>
 * If there are any dynamic parameters in any of the root nodes attached
 * to the canvas, the AnimCanvas will draw continuously.  If there are no
 * dynamic properties, the animation is suspended.  
 * <p>
 * <b>NOTE:</b>  There appears to be a bug in GL4Java, that multiple canvases
 * do not work correctly.  For now, do not open multiple windows until you know
 * your software works.  It appears to be a problem with the locking in the paint
 * method, which allows the OpenGL commands from the multiple windows to be 
 * interleaved.
 *
 * @version 	1.00
 * @author      Blair MacIntyre
 */ 
public class jooglCanvas extends GLAnimCanvas 
{
    /** the roots of the scene graphs for this window */
    protected Vector roots = new Vector();
    
    /** the clock for this window, shared by all roots */
    protected jooglClock clock = new jooglRealTimeClock();
    
    /** the width of this window */
    protected int width;
    /** the height of this window */
    protected int height;
            
    /**
     * A constructor that specifies the graphics configuration, capabilities, width and height.
     *
     * @param g The desired graphics configuration
     * @param glCaps The desired graphics capabilities
     * @param w The width of the window
     * @param h The height of the window
     */
    public jooglCanvas(GraphicsConfiguration g, GLCapabilities glCaps, int w, int h)
    {
        super(g, glCaps, w, h);
        height = h;
        width = w;
    }
    
    /**
     * A constructor that specifies the capabilities, width and height.
     *
     * @param glCaps The desired graphics capabilities
     * @param w The width of the window
     * @param h The height of the window
     */
    public jooglCanvas(GLCapabilities glCaps, int w, int h)
    {
        super(glCaps, w, h);
        height = h;
        width = w;
    }

    /**
     * A constructor that specifies the width and height.
     *
     * @param w The width of the window
     * @param h The height of the window
     */
    public jooglCanvas(int w, int h)
    {
        super(w, h);
        height = h;
        width = w;
    }

    /**
     * Set the clock for this window.
     * <p>
     * This is the clock that will be used by all jooglRoot objects to render into this window.
     *
     * @param c The new clock object
     */
    public void setClock(jooglClock c)
    {
        clock = c;
        repaint();
    }
    
    /**
     * Get the clock for this window.
     * <p>
     * This is the clock that is being used by all jooglRoot objects to render into this window.
     *
     * @return The current clock object
     */
    public jooglClock getClock()
    {
        return clock;
    }
    
    // methods to manipulate the collection of viewports.  
    // These are analogous to the Collection methods.
    
    /**
     * Add a jooglRoot to the canvas.
     *  <p>
     * Analogous to the add method on a Vector object.
     *
     * @param n the new root object
     * @return true if the collection of Roots changed as a result of the call
     */
    public boolean addRoot(jooglRoot n) 
    {
        boolean ret = roots.add(n);
        repaint();
        return ret;
    }
    
    /**
     * remove a jooglRoot from the canvas.
     *  <p>
     * Analogous to the remove method on a Collection object.
     *
     * @param n the root object to remove
     * @return true if the collection of Roots contained n
     */
    public boolean removeRoot(jooglRoot n) 
    {
        boolean ret = roots.remove(n);
        repaint();
        return ret;
    }
        
    /**
     * Remove all jooglRoot objects from the canvas.
     *  <p>
     * Analogous to the clear method on a Collection object.
     */
    public void clearRoots()
    {
        roots.clear();
        repaint();
    }
    
    /**
     * Test to see if a jooglRoot is in the canvas.
     *  <p>
     * Analogous to the contains method on a Collection object.
     *
     * @param n the root object
     * @return true if the collection of Roots contains n
     */
    public boolean containsRoot(jooglRoot n)
    {
        return roots.contains(n);
    }
        
    /**
     * Create an iterator over the collection of jooglRoots in the canvas.
     *  <p>
     * Analogous to the interator method on a Collection object.
     *
     * @return a new iterator on the collection of Root objects
     */
    public Iterator rootIterator() 
    {
        return roots.iterator();
    }


    /**
     * Test if there are any jooglRoots in the canvas.
     *  <p>
     * Analogous to the isEmpty method on a Collection object.
     *
     * @return true if there is at least one Roots in the canvas
     */
    public boolean isRoots()
    {
        return !roots.isEmpty();
    }
    
    /**
     * The number of roots in the canvas.
     *  <p>
     * Analogous to the size method on a Collection object.
     *
     * @return the number of Root objects in the canvas
     */
    public int numberOfRoots() 
    {
        return roots.size();
    }
        
    /**
    * Pick objects at position x,y in the root's window, with an
    * area of width/height around the mouse point.  
    * <p>
    * pickRoot() returns a Vector filled with the hit records for each object
    * hit.  If there are no objects hit, pickRoot() returns null. Each hit
    * record is a Vector, with the contents as follows: 
    * <pre>
       [jooglRoot, number of objects on the branch, zmin, zmax, o, p1, p2,...,pN] 
    * </pre>
    * o is the geometry object picked. The pi's are jooglPickable property nodes, 
    * and correspond to the pickable objects on the path
    * from the root node, starting with the object closest to the root (p1)
    * and ending with the object closest to geometry node (pN).   Only those pickable
    * properties with pickable set to true will be included.
    *
    * @param x  the x location of the selector (cursor)
    * @param y  the y location of the selector (cursor)
    * @param width  the width of the area around the cursor
    * @param height  the height of the area around the cursor
    */
    public Vector pickRoot (double x, double y, double width, double height)
    {
        return null;
    }
   
    /**
     * Initialize the canvas before creating the GLContext.
     * <p>
     * Called just BEFORE the GL-Context is created.
     * <p>
     * Define a reasonable set of 3D options of the jooglCanvas.  
     * Also, attempt to set the sharedGLContext here, so that all jooglCanvas
     * objects share textures and display lists.
     */
    public void preInit()
    {
        // We want double buffering
        doubleBuffer = true;
        // But we dont want stereo view
        stereoView = false; 

        // We want to share a glcontext, so display lists and textures are shared
        // we can call getSharedContext() on any joogl object
        sharedGLContext = clock.getSharedContext();
    }

    /**
     * Initialize the canvas after creating the GLContext.
     * <p>
     * Called just AFTER the GL-Context is created.   Set up the OpenGL state for joogl.
     */
    public void init()
    {
        width = getSize().width;
        height = getSize().height;
        
        // enable backface culling
        
        // set COUNTER CLOCKWISE ordering for front facing polygons

        // Set up Z buffering and enable it

        // Turn on automatic normal computation and automatic normalization of normals

        // It is suggested you use COLOR_MATERIAL mode for your lighting, so set that up
                        
        // set up the default viewing stacks

        // Enable lighting 

        // turn off all lights, just to be safe!

        // turn off the global ambient light, because we use explicit ambient lights

        // enable local viewer calculations on the lights

        // add this canvas to the list of known jooglCanvases.  Can use any joogl object to
        // get a handle on the addCanvas method.
        clock.addCanvas(this);

        // if we did not set a sharedGLContext, our canvas should be the shared context
        if (sharedGLContext == null)
            clock.setSharedContext(getGLContext());
    }
        
    /**
     * Destroy the canvas.
     */
    public void cvsDispose()
    {
        clock.removeCanvas(this);
    }
	
    /**
     * Called after the first paint command, and any window resize. 
     * <p>
     * In addition to keeping track of the window size, tell any root's to resize their viewports if necessary. 
     *
     * @param w the width of the window
     * @param h the height of the window
     */
    public void reshape(int w, int h)
    {
    }
        
    /**
     * Render all the jooglRoots.
     * <p>
     * Draw to the canvas.   All of the work to draw the jooglRoot objects should be done here.
     * 
     */
    public void display()
    {
        // Ensure GL is initialised correctly
        if (glj.gljMakeCurrent() == false)
            return;
        
        // System.out.println("display: " + glj);

        // Clear The Whole Window And The Depth Buffer
        gl.glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
        gl.glViewport(0, 0, width, height);
        gl.glClear(GL_COLOR_BUFFER_BIT| GL_DEPTH_BUFFER_BIT);
        
        long time = clock.getTime();
        
        Iterator i = roots.iterator();
        while (i.hasNext()) 
        {
            jooglRoot root = ((jooglRoot)(i.next()));

            // Do lots of stuff here!
        }
        
        // Swap buffers
        glj.gljSwap();
        glj.gljFree();
    }      
}
