/**
 * The abstract joogl light node.
 * <p>
 * This class contains all the information shared by all light objects.
 * Each light has a color, specified as an r,g,b triple.
 * Each light also has an intensity that is used to scale the color value.
 * The intensity defaults to 1.0.  
 *  <p>
 * Finally, each light can be turned on and off without changing the color or intensity.
 * <p>
 * For example, if a light has color 
 * (0.2,0.4,0.8) and an intensity of 2, the light's color ends up as (0.4,0.8,1.6).
 *
 * @version 	1.00
 * @author      Blair MacIntyre
 */

public abstract class jooglLight extends jooglNode {
    /** 
     * The red component of the light color.
     *  
     * @see #setColor
     * @see #getColor
     */
    protected float r;
    
    /** 
     * The green component of the light color.
     *  
     * @see #setColor
     * @see #getColor
     */
    protected float g;

    /** 
     * The blue component of the light color.
     *  
     * @see #setColor
     * @see #getColor
     */
    protected float b;

    /** 
     * The intensity scale factor for the color.
     *
     * @see #setIntensity
     * @see #getIntensity
     */
    protected float intensity;
    
    /**
     * Indicates if the light is on or off.
     *
     * @see #setLightOn
     * @see #getLightOn
     */
    protected boolean lightOn; 
    
    /**
     * Constructor function for an abstract light.
     * <p>
     * Specify the RGB values for the light color.  A default intensity of 1.0 is used.
     * 
     * @param r		the red component of the light
     * @param g		the green component of the light
     * @param b		the blue component of the light
     */
    public jooglLight(float _r, float _g, float _b)
    {
        super();
        r = _r;
        g = _g;
        b = _b;
        intensity = 1.0f;
        lightOn = true;
    }

    /**
     * Set the light color.
     * 
     * @param r		the red component of the light
     * @param g		the green component of the light
     * @param b		the blue component of the light
     */
    public void setColor(float _r, float _g, float _b)
    {
        r = _r;
        g = _g;
        b = _b;
    }
    
    /**
     * Set the light color.
     * 
     * @param c 	a three element vector containing the r, g, b color components of the light color
     */
    public void setColor(float[] c)
    {
        r = c[0];
        g = c[1];
        b = c[2];
    }
    
    /**
     * Get the light color.
     * 
     * @return	a three element vector containing the r, g, b color components of the light color
     */
    public float[] getColor()
    {
        float[] c = new float[3];
        c[0] = r;
        c[1] = g;
        c[2] = b;
        return c;
    }
    
    /**
     * Set the intensity of the light.
     *
     * @param i   the scale factor to be used to scale the light color by.
     */
    public void setIntensity(float i)
    {
        intensity = i;
    }
    
    /**
     * Get the current intensity of the light.
     *
     * @return   the current scale factor for the light color.
     */
    public float getIntensity()
    {
        return intensity;
    }

    /** 
     * Set the boolean flag indicating if the light is on or off.
     * <p>
     * If the flag is off, the light will not be used.  If it is on, it will be used whenever it appears in
     * the scene graph.
     *
     * @param on  indicates if the light is on or off
     */
    public void setLightOn(boolean on)
    {
        lightOn = on;
    }
    
    /** 
     * Get the boolean flag indicating if the light is on or off.
     *
     * @return  A boolean value indicating if the light is on or off
     */
    public boolean getLightOn()
    {
        return lightOn;
    }
    
    /**
     * Abstract method, overriden by each light object to create an OpenGL light as needed.
     * <p>
     * This method will be called each time a light is encountered in the scene graph to turn an
     * OpenGL light on based on the contents of the light node.   This method will only turn on
     * the light if its "lightOn" boolean is true, and will return the value of lightOn.
     *
     * @param lightNum	the OpenGL light number to use for this light.
     *
     @ @return a boolean indicating if the light is on or off.
     */
    protected abstract boolean activate(int lightNum);
}
