/**
 * The joogl perspective camera object.
 * <p>
 * This object creates a perspective camera, analogous to the standard OpenGL
 * gluPerspective and gluLookat commands.
 * <p>
 * The parameters correspond to what you would use in OpenGL to create a 
 * perspective projection:
 * <pre>
 fx,fy,fz: the viewpoint ("from")
 tx,ty,tz: the lookat point ("to")
 ux,uy,uz: the up vector ("up")
 fov: the field of view in the y direction
 near,far: the clipping planes
 * </pre>
 *
 * @version 	1.00
 * @author      Blair MacIntyre
 */
 
 
public class jooglPerspectiveCamera extends jooglCamera {
    /** the field of view of the camera, in degrees */
    protected double fovy; 
    
    /** 
     * A constructor for the perspective camera.
     * <p>
     * Create a perspective camera object to be used to view a scene graph.
     *
     * @param _fx  the x coordinate of the eyepoint
     * @param _fy  the y coordinate of the eyepoint
     * @param _fz  the z coordinate of the eyepoint
     * @param _tx  the x coordinate of the to point
     * @param _ty  the y coordinate of the to point
     * @param _tz  the z coordinate of the to point
     * @param _ux  the x coordinate of the up vector
     * @param _uy  the y coordinate of the up vector
     * @param _uz  the z coordinate of the up vector
     * @param _fovy  the field of view
     * @param _near  the near clipping plane
     * @param _far   the far clipping plane
     */
    public jooglPerspectiveCamera(double _fx, double _fy, double _fz, 
                        double _tx, double _ty, double _tz,
			double _ux, double _uy, double _uz,
			double _fovy, double _near, double _far)
    {
        super(_fx, _fy, _fz, _tx, _ty, _tz, _ux, _uy, _uz, _near, _far);
        fovy = _fovy;
    }

    /** 
     * A constructor for the perspective camera specifying only field of view.
     * <p>
     * Create a perspective camera using the default camera position and orientation
     * and a specified field of view.
     *
     * @param _fovy  the field of view
     */
    public jooglPerspectiveCamera(double _fovy)
    {
        super();
        fovy = _fovy;
    }
    
    /** 
     * A constructor for the perspective camera using default parameters.
     * <p>
     * Create a perspective camera using the default camera position and orientation
     * and a field of view of 35 degrees.
     */
    public jooglPerspectiveCamera()
    {
        super();
        fovy = 35.0;
    }
    
    /** 
     * Set the fovy of the camera.
     *
     * @param _fovy a double containing the field of view
     */
    public void setFovy(double _fovy)
    {
        fovy = _fovy;
    }
    
    /** 
     * Get the fovy of the camera.
     *
     * @return a double containing the field of view
     */
    public double getFovy()
    {
        return fovy;
    }
    
    /** 
     * Internal method, called to set the current PROJECTION to correspond to this camera.
     * <p>
     * This method sets up the camera, after transforming the parameters according to
     * the camCoords transformation.
     */
    protected void apply(jooglTransform camCoords)
    {
    }
}
